//
//  CalculatorViewController.m
//  Calculator
//
//  Created by Kevin Johnson on 7/31/11.
//  Copyright 2011 Home. All rights reserved.
//

#import "CalculatorViewController.h"
#import "CalculatorMemory.h"
#import "CalculatorEntry.h"

@implementation CalculatorViewController
@synthesize calculatorLabel;
@synthesize calculatorMemory;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    // An operation can only be selected when between numbers
    
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        
        performingOper = NO;
        allowDigitAppend = YES;
        decimalAdded = NO;
        
        self.calculatorMemory = [[CalculatorMemory alloc] init];
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

- (void)viewDidUnload
{
    [self setCalculatorLabel:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (IBAction)digitPressed:(UIButton *)sender {
    // Method that handles number buttons: 0 - 9
    
    if (allowDigitAppend) {

        // Remove leading 0 on cleared screen
        if ([self.calculatorLabel.text isEqualToString:CLEARED_SCREEN]) {
            self.calculatorLabel.text = @"";
        }
        
        // Append number to calculatorLabel
        self.calculatorLabel.text = [self.calculatorLabel.text stringByAppendingFormat:@"%@", sender.titleLabel.text];
        
        performingOper = NO;
        allowDigitAppend = YES;
        //decimalAdded = ; // Commented out since it wouldn't know if a decimal has been added
    }
}
- (IBAction)operPressed:(UIButton *)sender {
    // Method that handles operation buttons: +, -, *, /
    
    if (!performingOper) {
        self.calculatorLabel.text = [self.calculatorLabel.text stringByAppendingFormat:@" %@ ", sender.titleLabel.text];
        
        performingOper = YES;
        allowDigitAppend = YES;
        decimalAdded = NO;
    }
}

- (IBAction)equalsPressed:(UIButton *)sender {
    // Method that handles the equals button: =
    
    [self calculateResult];
    
    performingOper = NO;
    allowDigitAppend = NO;
    decimalAdded = NO;
}

- (IBAction)decimalPressed:(UIButton *)sender {
    // Method that handles the decimal button: .
    
    if (allowDigitAppend && !decimalAdded) {
        self.calculatorLabel.text = [self.calculatorLabel.text stringByAppendingFormat:@"%@", sender.titleLabel.text];
        
        decimalAdded = YES;
        allowDigitAppend = YES;
        performingOper = NO;
    }
}

- (IBAction)clearPressed:(id)sender {
    // Method that handles the clear button: C
    
    self.calculatorLabel.text = CLEARED_SCREEN;
    
    performingOper = NO;
    allowDigitAppend = YES;
    decimalAdded = NO;
}

- (IBAction)trigPressed:(UIButton *)sender {
    // Method that handles trig buttons: sin, cos, tan
    
    if (performingOper || [self.calculatorLabel.text isEqualToString:CLEARED_SCREEN]) {
        
        // Erase a cleared screen
        if ([self.calculatorLabel.text isEqualToString:CLEARED_SCREEN]) {
            self.calculatorLabel.text = @"";
        }
        
        self.calculatorLabel.text = [self.calculatorLabel.text stringByAppendingFormat:@" %@ ", sender.titleLabel.text];
        
        performingOper = YES;
        allowDigitAppend = YES;
        decimalAdded = NO;
    }
}

- (IBAction)memoryPressed:(UIButton *)sender {
    if ([sender.titleLabel.text isEqualToString:@"MS"]) {
        
        [calculatorMemory addCalculatorEntry:self.calculatorLabel.text];
    }
    else if ([sender.titleLabel.text isEqualToString:@"MR"]) {
        if(performingOper) {
            self.calculatorLabel.text = [self.calculatorLabel.text stringByAppendingFormat:@" %@", [calculatorMemory recallMemory]];
            
            performingOper = NO;
            allowDigitAppend = NO;
            decimalAdded = NO;
        }
        else {
            self.calculatorLabel.text = [calculatorMemory recallMemory];
        }
    }
    else if ([sender.titleLabel.text isEqualToString:@"MC"]) {
        [calculatorMemory clearMemory];
    }
}

- (void)calculateResult {
    // This will hold the string with calculated sin, cos, tan operations
    NSMutableString *partialComputedString = [[NSMutableString alloc] initWithString:@""];

    // Tokenize string
    NSArray *listItems = [self.calculatorLabel.text componentsSeparatedByString:@" "];
    
    // First compute any sin, cos, tan operations
    NSEnumerator *e = [listItems objectEnumerator];
    NSString *object;
    while (object = [e nextObject]) {
        if ([object isEqualToString:@"sin"]) {
            NSString *nextS;
            if (nextS = [e nextObject]) {
                [partialComputedString appendFormat:@" %f", sin([nextS doubleValue])];
            }
        }
        else if ([object isEqualToString:@"cos"]) {
            NSString *nextS;
            if (nextS = [e nextObject]) {
                [partialComputedString appendFormat:@" %f", cos([nextS doubleValue])];
            }
        }
        else if ([object isEqualToString:@"tan"]) {
            NSString *nextS;
            if (nextS = [e nextObject]) {
                [partialComputedString appendFormat:@" %f", tan([nextS doubleValue])];
            }
        }
        else {
            [partialComputedString appendFormat:@" %@", object];
        }
    }

    
    // Finish *, /, +, - calculations
    UIWebView *webView = [[UIWebView alloc] init];  // Move to global variable
    self.calculatorLabel.text = [webView stringByEvaluatingJavaScriptFromString:partialComputedString];
    [webView release];
    
    [partialComputedString release];
}





- (void)dealloc {
    [calculatorLabel release];
    [calculatorMemory release];
    [super dealloc];
}
@end
